package com.rshy.project.hy.config;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.AsyncConfigurer;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;

import java.util.concurrent.ThreadPoolExecutor;

/**
 * 线程池配置文件
 *
 * @see <a href="https://www.jianshu.com/p/5bf7691c1f1b" />
 *
 * @author fanqun
 * Created on 2022-05-10 11:17 AM
 **/
@Configuration
@EnableAsync
public class ThreadPoolTaskConfig implements AsyncConfigurer {

    /* 核心线程数（默认线程数） */
    private static final int CORE_POOL_SIZE = 18;
    /* 最大线程数 */
    private static final int MAX_POOL_SIZE = 20;
    /* 允许线程空闲时间（单位：默认为秒） */
    private static final int KEEP_ALIVE_TIME = 30;
    /* 缓冲队列数 */
    private static final int QUEUE_CAPACITY = 200;
    // 线程池名前缀
    private static final String THREAD_NAME_PREFIX = "HS-Async-";

    @Bean(name = "hstTaskExecutor") // bean的名称，默认为首字母小写的方法名
    public ThreadPoolTaskExecutor taskExecutor() {
        ThreadPoolTaskExecutor executor = new ThreadPoolTaskExecutor();
        executor.setCorePoolSize(CORE_POOL_SIZE);
        executor.setMaxPoolSize(MAX_POOL_SIZE);
        executor.setQueueCapacity(QUEUE_CAPACITY);
        executor.setKeepAliveSeconds(KEEP_ALIVE_TIME);
        executor.setThreadNamePrefix(THREAD_NAME_PREFIX);
        executor.setWaitForTasksToCompleteOnShutdown(true);
        executor.setAwaitTerminationSeconds(60);
        // 线程池对拒绝任务的处理策略
        executor.setRejectedExecutionHandler(new ThreadPoolExecutor.CallerRunsPolicy());
        // 初始化
        executor.initialize();
        return executor;
    }
}
